﻿namespace Excercise01
{
    internal class Excercise02
    {
        /// <summary> Bitmap image used to display our drawings </summary>
        Bitmap canvas;
        int width;
        int height;
        /// <summary> Graphics context used for drawing </summary>
        Graphics g;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="canvas"> Canvas - bitmap image </param>
        public Excercise02(Bitmap canvas)
        {
            this.canvas = canvas;
            width = canvas.Width;
            height = canvas.Height;

            g = Graphics.FromImage(canvas);
            ClearImage();
        }

        /// <summary>
        /// Paint whole image grey
        /// </summary>
        void ClearImage()
        {
            g.Clear(Color.FromArgb(42, 42, 42));
        }

        /// <summary>
        /// Generate a random maze and draw it
        /// </summary>
        /// <param name="m"> Number of rows </param>
        /// <param name="n"> Number of columns </param>
        /// <param name="cellsize"> Size of one cell </param>
        public void GenerateMaze(int m, int n, int cellsize)
        {
            // create a new maze and generate it
            Maze maze = new Maze();
            maze.GenerateMaze(m, n);
            // draw all cells
            DrawMaze(maze, cellsize);
        }

        /// <summary>
        /// Draw generated maze
        /// </summary>
        /// <param name="maze"> Maze to draw </param>
        /// <param name="cellsize"> Size of one cell </param>
        private void DrawMaze(Maze maze, int cellsize)
        {

            ClearImage();
            Pen p = new Pen(Color.White, 2);

            // go through all cells
            for (int i = 0; i < maze.cells.GetLength(0); i++)
            {
                for (int j = 0; j < maze.cells.GetLength(1); j++)
                {
                    Point pNW = new Point(cellsize + j * cellsize, cellsize + i * cellsize);
                    Point pNE = new Point(cellsize + (j + 1) * cellsize, cellsize + i * cellsize);
                    Point pSW = new Point(cellsize + j * cellsize, cellsize + (i + 1) * cellsize);

                    // draw north and west walls if they exist
                    if (maze.cells[i, j].north)
                        g.DrawLine(p, pNW, pNE);
                    if (maze.cells[i, j].west)
                        g.DrawLine(p, pNW, pSW);
                }
            }

            // draw south and east side of the maze
            Point NE = new Point(cellsize + maze.cells.GetLength(1) * cellsize, cellsize);
            Point SE = new Point(cellsize + maze.cells.GetLength(1) * cellsize,
                                 cellsize + maze.cells.GetLength(0) * cellsize);
            Point SW = new Point(cellsize, cellsize + maze.cells.GetLength(0) * cellsize);
            g.DrawLine(p, NE, SE);
            g.DrawLine(p, SE, SW);
        }
    }
}
